module Misfortune
  module CLI
    class BingeCommand
      attr_reader :options, :cookies, :flavor

      def initialize(options)
        @options = options
        @cookies = CookieJar.new(options['source']).cookies
        @flavor = Flavor.new(:terminal)

        trap_interrupt
      end

      def run
        current = 0
        loop do
          render(cookies[current % cookies_count])
          current = select_cookie(current)
        end
      end

      protected

      def cookies_count
        @cookies_count ||= cookies.count
      end

      def render(cookie)
        system 'clear'
        puts flavor.apply_to(cookie)
        puts "\n\nPress ENTER to continue\n\n"
      end

      def select_cookie(current)
        selection = STDIN.gets.chomp.to_i
        selection > 0 ? selection : current + 1
      end

      def trap_interrupt
        Signal.trap('INT') do
          exit
        end
      end
    end
  end
end
